==> pan-0.6.5.tar.bz2: pan-0.6.5/pan/date.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "pan.h"

Timezone_Table_Type Timezone_Table[] = {
	{"EDT", -500},
	{"PST", -800},
	{"CST", -600},
	{"MST", -700},
	{"MET", 100},				/* Middle European */
	{"MEZ", 100},				/* Middle European */
	{"MSK", 300},				/* Moscow */
	{"CET", 100},				/* Central European */
	{"HKT", 800},				/* Central European */
	{"JST", 900},				/* Central European */
	{"CAST", 930},				/* Central Autsralian */
	{"EAST", 1000},				/* Eastern Autsralian */
	{"NZST", 1200},				/* New Zealand Autsralian */
	{"EET", 200},				/* Eastern European */
	{"", 0}
};


gchar *
skip_whitespace (gchar *b)
{
	register char ch;

	if (b == NULL)
		return NULL;

	while (((ch = *b) == ' ') || (ch == '\t') || (ch == '\n'))
		b++;
	return b;
}


int
parse_timezone (gchar *t)
{
	gchar ch;
	Timezone_Table_Type *table;

	table = Timezone_Table;

	while (0 != (ch = table->name[0])) {
		if ((*t == ch) && (0 == strncmp(t, table->name, sizeof(table->name))))
			return table->tz;
		table++;
	}

	return 0;
}


time_t
parse_date (char *date)
{
	char ch;
	long day, month, year, minutes, seconds, hours, tz;
	int sign;

	date = skip_whitespace(date);
	if ((date == NULL) || (*date == 0))
		return 0;

	/* Look for a weekday, if found skip it */
	while (isalpha((int)*date))
		date++;
	if (*date == ',')
		date++;

	date = skip_whitespace(date);

	/* expecting "03 Feb 1997 10:09:58 +0100" */
	day = 0;
	while (ch = *date, isdigit((int)ch)) {
		day = 10 * day + (long) (ch - '0');
		date++;
	}
	if ((day == 0) || (day > 31))
		return 0;
	day--;

	date = skip_whitespace(date);
	month = 0;
	switch (*date++) {
	default:
		return 0;
	case 'J':					/* Jan, Jun, Jul */
		ch = *date++;
		if ((ch == 'u') || (ch == 'U')) {
			ch = *date++;
			if ((ch == 'n') || (ch == 'N'))
				month = 6;
			else if ((ch == 'l') || (ch == 'L'))
				month = 7;
		} else if ((ch == 'a') || (ch == 'A'))
			month = 1;
		break;

	case 'F':
	case 'f':					/* Feb */
		month = 2;
		break;
	case 'M':
	case 'm':					/* May, Mar */
		ch = *date++;
		if ((ch == 'a') || (ch == 'A')) {
			ch = *date++;
			if ((ch == 'y') || (ch == 'Y'))
				month = 5;
			else if ((ch == 'r') || (ch == 'R'))
				month = 3;
		}
		break;
	case 'A':
	case 'a':					/* Apr, Aug */
		ch = *date++;
		if ((ch == 'p') || (ch == 'P'))
			month = 4;
		else if ((ch == 'u') || (ch == 'U'))
			month = 8;
		break;

	case 'S':
	case 's':					/* Sep */
		month = 9;
		break;
	case 'O':
	case 'o':					/* Oct */
		month = 10;
		break;
	case 'N':
	case 'n':					/* Nov */
		month = 11;
		break;
	case 'D':
	case 'd':					/* Dec */
		month = 12;
		break;
	}

	if (month == 0)
		return 0;

	month--;

	/* skip past month onto year. */
	while (isalpha((int)*date))
		date++;
	date = skip_whitespace(date);

	year = 0;
	if (*date == '9')
		year = 19;

	while (ch = *date, isdigit((int)ch)) {
		year = year * 10 + (long) (ch - '0');
		date++;
	}
	date = skip_whitespace(date);

	/* Now parse hh:mm:ss */
	hours = 0;
	while (ch = *date, isdigit((int)ch)) {
		hours = hours * 10 + (long) (ch - '0');
		date++;
	}
	if ((ch != ':') || (hours >= 24))
		return 0;
	date++;

	minutes = 0;
	while (ch = *date, isdigit((int)ch)) {
		minutes = minutes * 10 + (long) (ch - '0');
		date++;
	}
	if (minutes >= 60)
		return 0;

	/* Seconds may not be present */
	seconds = 0;
	if (ch == ':') {
		date++;
		while (ch = *date, isdigit((int)ch)) {
			seconds = seconds * 10 + (long) (ch - '0');
			date++;
		}
		if (seconds >= 60)
			return 0;
	}
	/* Now timezone */
	date = skip_whitespace(date);

	sign = 1;
	if (*date == '+')
		date++;
	else if (*date == '-') {
		sign = -1;
		date++;
	}
	tz = 0;
	while (ch = *date, isdigit((int)ch)) {
		tz = tz * 10 + (long) (ch - '0');
		date++;
	}
	tz = sign * tz;

	date = skip_whitespace(date);
	if (isalpha((int)*date)) {
		sign = 1;
		if ((*date != 'G') && (*date != 'M') && (*date != 'T'))
			tz = parse_timezone(date);
	}
	/* Compute the number of days since beginning of year. */
	day = 31 * month + day;
	if (month > 1) {
		day -= (month * 4 + 27) / 10;
		if (IS_LEAP_YEAR(year))
			day++;
	}
	/* add that to number of days since beginning of time */
	year -= THE_YEAR_0;
	day += year * 365 + year / 4;
	if ((year % 4) == 1)
		day++;

	/* Adjust hours for timezone */
	hours -= tz / 100;
	minutes -= tz % 100;		/* ?? */


	/* Now convert to secs */
	seconds += 60L * (minutes + 60L * (hours + 24L * day));

	return seconds;
}


gchar *
generate_date (void)
{
	time_t secs;
	struct tm *gmt;
	gchar *date;

	static const char *days[] = {
		"Sun", "Mon", "Tue", "Wed",
		"Thu", "Fri", "Sat",
	};
	static const char *months[] = {
		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
	};

	time(&secs);
	gmt = gmtime(&secs);

	date = g_malloc(sizeof(gchar) * 28);

	sprintf (date, "%s, %2d %s %2d %02d:%02d:%02d GMT",
		 days[gmt->tm_wday],
		 gmt->tm_mday,
		 months[gmt->tm_mon],
		 gmt->tm_year, gmt->tm_hour, gmt->tm_min, gmt->tm_sec);

	return date;
}

==> pan-0.6.6.tar.bz2: pan-0.6.6/pan/date.c <==
/* Pan - Date gemerating and conversion
 *
 * Copyright (C) 1999 Matt Eagleson (e.Messiah@superpimp.org)
 *
 * Authors: Matt Eagleson <e.Messiah@superpimp.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "pan.h"


typedef struct
{
	char name[5];
	int tz;
}
Timezone_Table_Type;


Timezone_Table_Type Timezone_Table[] = {
	{"EDT", -500},
	{"PST", -800},
	{"CST", -600},
	{"MST", -700},
	{"MET", 100},				/* Middle European */
	{"MEZ", 100},				/* Middle European */
	{"MSK", 300},				/* Moscow */
	{"CET", 100},				/* Central European */
	{"HKT", 800},				/* Central European */
	{"JST", 900},				/* Central European */
	{"CAST", 930},				/* Central Autsralian */
	{"EAST", 1000},				/* Eastern Autsralian */
	{"NZST", 1200},				/* New Zealand Autsralian */
	{"EET", 200},				/* Eastern European */
	{"", 0}
};


static const gchar*
skip_whitespace (const gchar *b)
{
	register char ch;

	if (b == NULL)
		return NULL;

	while (((ch = *b) == ' ') || (ch == '\t') || (ch == '\n'))
		b++;
	return b;
}


static int
parse_timezone (const gchar *t)
{
	gchar ch;
	Timezone_Table_Type *table;

	table = Timezone_Table;

	while (0 != (ch = table->name[0])) {
		if ((*t == ch) && (0 == strncmp(t, table->name, sizeof(table->name))))
			return table->tz;
		table++;
	}

	return 0;
}


time_t
parse_date (const char *date)
{
	char ch;
	long day, month, year, minutes, seconds, hours, tz;
	int sign;

	date = skip_whitespace(date);
	if ( !date || !*date )
		return 0;

	/* Look for a weekday, if found skip it */
	while (isalpha((int)*date))
		date++;
	if (*date == ',')
		date++;

	date = skip_whitespace(date);

	/* expecting "03 Feb 1997 10:09:58 +0100" */
	day = 0;
	while (ch = *date, isdigit((int)ch)) {
		day = 10 * day + (long) (ch - '0');
		date++;
	}
	if ((day == 0) || (day > 31))
		return 0;
	day--;

	date = skip_whitespace(date);
	month = 0;
	switch (*date++) {
	default:
		return 0;
	case 'J':					/* Jan, Jun, Jul */
		ch = *date++;
		if ((ch == 'u') || (ch == 'U')) {
			ch = *date++;
			if ((ch == 'n') || (ch == 'N'))
				month = 6;
			else if ((ch == 'l') || (ch == 'L'))
				month = 7;
		} else if ((ch == 'a') || (ch == 'A'))
			month = 1;
		break;

	case 'F':
	case 'f':					/* Feb */
		month = 2;
		break;
	case 'M':
	case 'm':					/* May, Mar */
		ch = *date++;
		if ((ch == 'a') || (ch == 'A')) {
			ch = *date++;
			if ((ch == 'y') || (ch == 'Y'))
				month = 5;
			else if ((ch == 'r') || (ch == 'R'))
				month = 3;
		}
		break;
	case 'A':
	case 'a':					/* Apr, Aug */
		ch = *date++;
		if ((ch == 'p') || (ch == 'P'))
			month = 4;
		else if ((ch == 'u') || (ch == 'U'))
			month = 8;
		break;

	case 'S':
	case 's':					/* Sep */
		month = 9;
		break;
	case 'O':
	case 'o':					/* Oct */
		month = 10;
		break;
	case 'N':
	case 'n':					/* Nov */
		month = 11;
		break;
	case 'D':
	case 'd':					/* Dec */
		month = 12;
		break;
	}

	if (month == 0)
		return 0;

	month--;

	/* skip past month onto year. */
	while (isalpha((int)*date))
		date++;
	date = skip_whitespace(date);

	year = 0;
	if (*date == '9')
		year = 19;

	while (ch = *date, isdigit((int)ch)) {
		year = year * 10 + (long) (ch - '0');
		date++;
	}
	date = skip_whitespace(date);

	/* Now parse hh:mm:ss */
	hours = 0;
	while (ch = *date, isdigit((int)ch)) {
		hours = hours * 10 + (long) (ch - '0');
		date++;
	}
	if ((ch != ':') || (hours >= 24))
		return 0;
	date++;

	minutes = 0;
	while (ch = *date, isdigit((int)ch)) {
		minutes = minutes * 10 + (long) (ch - '0');
		date++;
	}
	if (minutes >= 60)
		return 0;

	/* Seconds may not be present */
	seconds = 0;
	if (ch == ':') {
		date++;
		while (ch = *date, isdigit((int)ch)) {
			seconds = seconds * 10 + (long) (ch - '0');
			date++;
		}
		if (seconds >= 60)
			return 0;
	}
	/* Now timezone */
	date = skip_whitespace(date);

	sign = 1;
	if (*date == '+')
		date++;
	else if (*date == '-') {
		sign = -1;
		date++;
	}
	tz = 0;
	while (ch = *date, isdigit((int)ch)) {
		tz = tz * 10 + (long) (ch - '0');
		date++;
	}
	tz = sign * tz;

	date = skip_whitespace(date);
	if (isalpha((int)*date)) {
		sign = 1;
		if ((*date != 'G') && (*date != 'M') && (*date != 'T'))
			tz = parse_timezone(date);
	}
	/* Compute the number of days since beginning of year. */
	day = 31 * month + day;
	if (month > 1) {
		day -= (month * 4 + 27) / 10;
		if (IS_LEAP_YEAR(year))
			day++;
	}
	/* add that to number of days since beginning of time */
	year -= THE_YEAR_0;
	day += year * 365 + year / 4;
	if ((year % 4) == 1)
		day++;

	/* Adjust hours for timezone */
	hours -= tz / 100;
	minutes -= tz % 100;		/* ?? */


	/* Now convert to secs */
	seconds += 60L * (minutes + 60L * (hours + 24L * day));

	return seconds;
}


/*---[ generate_date ]------------------------------------------------
 * generate a properly formatted Date: for a message being sent to
 * an NNTP server
 *--------------------------------------------------------------------*/
gchar *
generate_date (void)
{
	time_t secs;
	struct tm *gmt;
	gchar *date;

	static const char *days[] = {
		"Sun", "Mon", "Tue", "Wed",
		"Thu", "Fri", "Sat",
	};
	static const char *months[] = {
		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
	};

	time(&secs);
	gmt = gmtime(&secs);

	date = g_malloc(sizeof(gchar) * 28);

	sprintf (date, "%s, %2d %s %2d %02d:%02d:%02d GMT",
		 days[gmt->tm_wday],
		 gmt->tm_mday,
		 months[gmt->tm_mon],
		 gmt->tm_year, gmt->tm_hour, gmt->tm_min, gmt->tm_sec);

	return date;
}

/* unused code - JEL
gchar *
generate_newgroups_date (void)
{
	struct tm *gmt;
	gchar *date;
	time_t secs;

	time (&secs);
	gmt = gmtime (&secs);

	date = g_malloc (sizeof(gchar) * 18);

	sprintf (date, "%02d%02d%02d %02d%02d%02d GMT",
		 gmt->tm_year, gmt->tm_mon+1, gmt->tm_mday,
		 gmt->tm_hour, gmt->tm_min, gmt->tm_sec);

	return date;
}
*/
